"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const util = require("util");
const chalk_1 = require("chalk");
const lodash = require("lodash");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
class ConfigGetCommand extends command_1.Command {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'get',
                type: 'global',
                description: 'Print config values',
                longDescription: `
By default, this command prints properties in your project's ${chalk_1.default.bold(cli_utils_1.PROJECT_FILE)} file.

For ${chalk_1.default.green('--global')} config, the CLI prints properties in the global CLI config file (${chalk_1.default.bold('~/.ionic/config.json')}).

For nested properties, separate nest levels with dots. For example, the property name ${chalk_1.default.green('user.email')} will look in the ${chalk_1.default.bold('user')} object (a root-level field in the global CLI config file) for the ${chalk_1.default.bold('email')} field.

Without a ${chalk_1.default.green('property')} argument, this command prints out the entire file contents.

If you are using this command programmatically, you can use the ${chalk_1.default.green('--json')} option.

This command attempts to sanitize config output for known sensitive fields, such as fields within the ${chalk_1.default.bold('tokens')} object in the global CLI config file. This functionality is disabled when using ${chalk_1.default.green('--json')}.
      `,
                inputs: [
                    {
                        name: 'property',
                        description: 'The property name you wish to get',
                    },
                ],
                options: [
                    {
                        name: 'global',
                        description: 'Use global CLI config',
                        type: Boolean,
                        aliases: ['g'],
                    },
                    {
                        name: 'json',
                        description: 'Output config values in JSON',
                        type: Boolean,
                    },
                ],
                exampleCommands: ['', 'app_id', '--global user.email', '-g yarn'],
            };
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [p] = inputs;
            const { global, json } = options;
            if (!global && !this.env.project.directory) {
                throw new errors_1.FatalException(`Sorry--this won't work outside an Ionic project directory. Did you mean to print global config using ${chalk_1.default.green('--global')}?`);
            }
            const file = global ? this.env.config : this.env.project;
            const config = yield file.load();
            const v = lodash.cloneDeep(p ? lodash.get(config, p) : config);
            if (json) {
                process.stdout.write(JSON.stringify(v));
            }
            else {
                this.sanitize(p, v);
                this.env.log.rawmsg(util.inspect(v, { colors: chalk_1.default.enabled }));
            }
        });
    }
    scrubTokens(obj) {
        return lodash.mapValues(obj, () => '*****');
    }
    sanitize(key, obj) {
        if (typeof obj === 'object' && 'tokens' in obj) {
            obj['tokens'] = this.scrubTokens(obj['tokens']);
        }
        if (key === 'tokens') {
            lodash.assign(obj, this.scrubTokens(obj));
        }
    }
}
exports.ConfigGetCommand = ConfigGetCommand;
