"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const lodash = require("lodash");
const format_1 = require("@ionic/cli-framework/utils/format");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const path_1 = require("@ionic/cli-framework/utils/path");
const cli_utils_1 = require("@ionic/cli-utils");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const base_1 = require("./base");
const DEFAULT_BITS = '2048';
const DEFAULT_COUNTRY_NAME = 'US';
const DEFAULT_STATE_OR_PROVINCE_NAME = 'Wisconsin';
const DEFAULT_LOCALITY_NAME = 'Madison';
const DEFAULT_ORGANIZATION_NAME = 'Ionic';
const DEFAULT_COMMON_NAME = 'localhost';
class SSLGenerateCommand extends base_1.SSLBaseCommand {
    getDefaultPath() {
        return this.env.project.directory ? path.resolve(this.env.project.directory, '.ionic/ssl') : path.resolve(this.env.config.directory, 'ssl');
    }
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'generate',
                type: 'global',
                description: 'Generates an SSL key & certificate',
                // TODO: document how to add trusted certs
                longDescription: `
Uses OpenSSL to create a self-signed certificate for ${chalk_1.default.bold('localhost')} (by default).

After the certificate is generated, you will still need to add it to your system or browser as a trusted certificate.

The default directory for ${chalk_1.default.green('--key-path')} and ${chalk_1.default.green('--cert-path')} is ${chalk_1.default.green('.ionic/ssl/')} inside of Ionic projects and ${chalk_1.default.green('~/.ionic/ssl/')} outside of Ionic projects.
      `,
                options: [
                    {
                        name: 'key-path',
                        description: 'Destination of private key file',
                        default: format_1.prettyPath(path.resolve(this.getDefaultPath(), 'key.pem')),
                    },
                    {
                        name: 'cert-path',
                        description: 'Destination of certificate file',
                        default: format_1.prettyPath(path.resolve(this.getDefaultPath(), 'cert.pem')),
                    },
                    {
                        name: 'country-name',
                        description: 'The country name (C) of the SSL certificate',
                        default: DEFAULT_COUNTRY_NAME,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'state-or-province-name',
                        description: 'The state or province name (ST) of the SSL certificate',
                        default: DEFAULT_STATE_OR_PROVINCE_NAME,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'locality-name',
                        description: 'The locality name (L) of the SSL certificate',
                        default: DEFAULT_LOCALITY_NAME,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'organization-name',
                        description: 'The organization name (O) of the SSL certificate',
                        default: DEFAULT_ORGANIZATION_NAME,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'common-name',
                        description: 'The common name (CN) of the SSL certificate',
                        default: DEFAULT_COMMON_NAME,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'bits',
                        description: 'Number of bits in the key',
                        aliases: ['b'],
                        default: DEFAULT_BITS,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                ],
                groups: [cli_utils_1.CommandGroup.Beta],
            };
        });
    }
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.checkForOpenSSL();
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const keyPath = format_1.expandPath(options['key-path'] ? String(options['key-path']) : path.join(this.getDefaultPath(), 'key.pem'));
            const keyPathDir = path.dirname(keyPath);
            const certPath = format_1.expandPath(options['cert-path'] ? String(options['cert-path']) : path.join(this.getDefaultPath(), 'cert.pem'));
            const certPathDir = path.dirname(certPath);
            const bits = options['bits'] ? String(options['bits']) : DEFAULT_BITS;
            const countryName = options['country-name'] ? String(options['country-name']) : DEFAULT_COUNTRY_NAME;
            const stateOrProvinceName = options['state-or-province-name'] ? String(options['state-or-province-name']) : DEFAULT_STATE_OR_PROVINCE_NAME;
            const localityName = options['locality-name'] ? String(options['locality-name']) : DEFAULT_LOCALITY_NAME;
            const organizationName = options['organization-name'] ? String(options['organization-name']) : DEFAULT_ORGANIZATION_NAME;
            const commonName = options['common-name'] ? String(options['common-name']) : DEFAULT_COMMON_NAME;
            yield this.ensureDirectory(keyPathDir);
            yield this.ensureDirectory(certPathDir);
            const overwriteKeyPath = yield this.checkExistingFile(keyPath);
            const overwriteCertPath = yield this.checkExistingFile(certPath);
            if (overwriteKeyPath) {
                yield fs_1.fsUnlink(keyPath);
            }
            if (overwriteCertPath) {
                yield fs_1.fsUnlink(certPath);
            }
            const cnf = { bits, countryName, stateOrProvinceName, localityName, organizationName, commonName };
            const cnfPath = yield this.writeConfig(cnf);
            yield this.env.shell.run('openssl', ['req', '-x509', '-newkey', `rsa:${bits}`, '-nodes', '-subj', this.formatSubj(cnf), '-reqexts', 'SAN', '-extensions', 'SAN', '-config', cnfPath, '-days', '365', '-keyout', keyPath, '-out', certPath], {});
            this.env.log.nl();
            this.env.log.ok(`Generated key & certificate!\n` +
                `Key:  ${chalk_1.default.bold(keyPath)}\n` +
                `Cert: ${chalk_1.default.bold(certPath)}`);
        });
    }
    formatSubj(cnf) {
        const subjNames = new Map([
            ['countryName', 'C'],
            ['stateOrProvinceName', 'ST'],
            ['localityName', 'L'],
            ['organizationName', 'O'],
            ['commonName', 'CN'],
        ]);
        return '/' + lodash.toPairs(cnf).filter(([k]) => subjNames.has(k)).map(([k, v]) => `${subjNames.get(k)}=${v}`).join('/');
    }
    ensureDirectory(p) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!(yield fs_1.pathExists(p))) {
                yield fs_1.fsMkdirp(p, 0o700);
                this.env.log.msg(`Created ${chalk_1.default.bold(format_1.prettyPath(p))} directory for you.\n`);
            }
        });
    }
    checkExistingFile(p) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (yield fs_1.pathExists(p)) {
                const confirm = yield this.env.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: `Key ${chalk_1.default.bold(format_1.prettyPath(p))} exists. Overwrite?`,
                });
                if (confirm) {
                    return true;
                }
                else {
                    throw new errors_1.FatalException(`Not overwriting ${chalk_1.default.bold(format_1.prettyPath(p))}.`);
                }
            }
        });
    }
    writeConfig({ bits, countryName, stateOrProvinceName, localityName, organizationName, commonName }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cnf = `
[req]
default_bits       = ${bits}
distinguished_name = req_distinguished_name

[req_distinguished_name]
countryName                = ${countryName}
stateOrProvinceName        = ${stateOrProvinceName}
localityName               = ${localityName}
organizationName           = ${organizationName}
commonName                 = ${commonName}

[SAN]
subjectAltName=DNS:${commonName}
`.trim();
            const p = yield path_1.tmpfilepath('ionic-ssl');
            yield fs_1.fsWriteFile(p, cnf, { encoding: 'utf8' });
            return p;
        });
    }
}
exports.SSLGenerateCommand = SSLGenerateCommand;
